/* --------------------------------------------------------------------------
 *
 * License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyright (c) 2001 Schuyler Heath <sheath@jabber.org>
 *
 * Acknowledgements
 *
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 *
 * --------------------------------------------------------------------------*/

#include "node.h"

/* catches <route type="error" to="sid"/> from c2s to JSM */
result node_jsm_packet_in(fni ni, dpacket d, void *arg)
{
    char *type;

    if (d->type == p_ROUTE &&
        (type = xmlnode_get_attrib(d->x,"type")) != NULL &&
        *type == 'e') {
        node_session_end(ni,d->id);
    }
    return r_PASS;
}

/* examin route packets out from JSM to c2s to determine when a session starts */
void node_jsm_check_route(fni ni, dpacket d)
{
    char *type;

    /* pthsock sends a <route type="error"/> after it receives a <route type="error"/>
       from JSM.  so we don't need to check for error here. */
    if ((type = xmlnode_get_attrib(d->x,"type")) != NULL &&
        *type == 's') {
        /* JSM acked the session start from c2s */
        node_session_start(ni,jid_new(d->p,xmlnode_get_attrib(d->x,"from")));
    }
}

/* packets from JSM to us */
result node_jsm_packet_deliver(fni ni, dpacket d, void *arg)
{
    char *id = (char *) arg;

    if (d->type == p_ROUTE)
    {
        if (xmlnode_get_attrib(d->x,"from") == NULL && j_strcmp(id,d->id->server) == 0)
        {
            node_inter_route(ni,xmlnode_get_firstchild(d->x));
            return r_DONE;
        }

        node_jsm_check_route(ni,d);
    }
    return r_PASS; /* deliver it on to the master component */
}

void node_jsm(fni ni)
{
    node_register(ni,PACKET_DELIVER,node_jsm_packet_deliver,(void *) ni->i->id);
    node_register(ni,PACKET_FARM,node_jsm_packet_in,NULL);
}
