/* --------------------------------------------------------------------------
 *
 * License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyright (c) 2001 Schuyler Heath <sheath@jabber.org>
 *
 * Acknowledgements
 *
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 *
 * --------------------------------------------------------------------------*/

#include "node.h"

typedef struct
{
    pool p;
    char *id;
    int state;
    time_t start;
} *tses, _tses;

typedef struct
{
    xht sessions;
    xht waiting;
    time_t delay;
} *ntrans, _ntrans;

#define s_RUNNING 1

void node_trans_deliver_iqreg(fni ni, dpacket d, void *arg)
{
    ntrans nt = (ntrans) arg;
    jid id = d->id;
    char *type;

    type = xmlnode_get_attrib(d->x,"type");
    if (/*j_strcmp(type,"result") == 0 */ 0)
    { /* XXX should this actually start a session? */
        tses s = (tses) xhash_get(nt->waiting,jid_full(id));
        if (0)
        {
            s->state = s_RUNNING;
            xhash_zap(nt->waiting,jid_full(id));
            node_session_start(ni,id);
        }
    }
    else if (j_strcmp(type,"error") == 0)
    {
        tses s = (tses) xhash_get(nt->waiting,jid_full(id));
        if (s)
        {
            /* remove pending */
            xhash_zap(nt->waiting,jid_full(id));
            xhash_zap(nt->sessions,jid_full(id));
            pool_free(s->p);
        }
    }
}

void node_trans_deliver_pres(fni ni, dpacket d, void *arg)
{
    ntrans nt = (ntrans) arg;
    jid id = jid_user(d->id);
    char *type;

    type = xmlnode_get_attrib(d->x,"type");
    if (type == NULL)
    {
        tses s = xhash_get(nt->waiting,jid_full(id));

        if (s != NULL)
        {
            log_debug(ZONE,"Starting session");
            xhash_zap(nt->waiting,jid_full(id));
            s->state = s_RUNNING;
            node_session_start(ni,id);
        }
    }
    else if (j_strcmp(type,"unavailable")==0)
    {
        tses s = (tses) xhash_get(nt->sessions,jid_full(id));

        if (s != NULL)
        {
            if (s->state == s_RUNNING)
                node_session_end(ni,id);
            else
                xhash_zap(nt->waiting,jid_full(id));
            xhash_zap(nt->sessions,jid_full(id));
            pool_free(s->p);
        }
        else
        {
            log_debug(ZONE,"Unknown session?");
        }
    }
}

/* packets from transport to user */
result node_trans_deliver(fni ni, dpacket d, void *arg)
{
    if (d->type == p_NORM)
    {
        jid from = jid_new(d->p,xmlnode_get_attrib(d->x,"from"));
        if (from->user != NULL) return r_PASS;

        if (*xmlnode_get_name(d->x) == 'p')
        {
            node_trans_deliver_pres(ni,d,arg);
        }
        else if (*xmlnode_get_name(d->x) == 'i')
        {
            node_trans_deliver_iqreg(ni,d,arg);
        }
    }
    return r_PASS;
}

void node_trans_start(ntrans nt, jid id)
{
    tses s;
    pool p;

    p = pool_new();
    s = pmalloco(p,sizeof(_tses));

    s->p = p;
    s->id = pstrdup(p,jid_full(id));
    s->start = time(NULL);

    xhash_put(nt->waiting,s->id,(void *) s);
    xhash_put(nt->sessions,s->id,(void *) s);
}

void node_trans_farm_iqreg(fni ni, dpacket d, void *arg)
{
    ntrans nt = (ntrans) arg;
    char *type = xmlnode_get_attrib(d->x,"type");

    if (type && *type == 's') /* register set */
    {
        /* session start or ignore */
        jid id = jid_user(jid_new(d->p,xmlnode_get_attrib(d->x,"from")));

        if (id->user != NULL && xhash_get(nt->sessions,jid_full(id)) == NULL)
            node_trans_start(nt,id);
    }
}

void node_trans_farm_pres(fni ni,dpacket d, void *arg)
{
    ntrans nt = (ntrans) arg;
    char *type;

    type = xmlnode_get_attrib(d->x,"type");
    if (type == NULL)
    {
        jid id = jid_user(jid_new(d->p,xmlnode_get_attrib(d->x,"from")));

        if (id->user != NULL && xhash_get(nt->sessions,jid_full(id)) == NULL)
            node_trans_start(nt,id);
    }
}

/* packets from user to transport node */
result node_trans_farm(fni ni, dpacket d, void *arg)
{
    if (d->type == p_NORM)
    {
        if (d->id->user != NULL) return r_PASS; /* ignore packets to users */

        if (*xmlnode_get_name(d->x) == 'p')
        {
            node_trans_farm_pres(ni,d,arg);
        }
        else if (*xmlnode_get_name(d->x) == 'i' && xmlnode_get_tag(d->x,"query?xmlns=jabber:iq:register"))
        {
            node_trans_farm_iqreg(ni,d,arg);
        }
    }
    return r_PASS; /* let it be delivered to the transport */
}

void node_clear_walk(xht h, const char *key, void *val, void *arg)
{
    ntrans nt = (ntrans) arg;
    tses s = (tses) val;

    if (s->start + nt->delay > time(NULL))
    {
        /* remove pending */
        xhash_zap(nt->waiting,s->id);
        xhash_zap(nt->sessions,s->id);
        pool_free(s->p);
    }
}

/* XXX this would be better as a list ordered by delay time left */
result node_clear(void *arg)
{
    ntrans nt = (ntrans) arg;
    xhash_walk(nt->waiting,node_clear_walk,(void *) nt);
    return r_DONE;
}

void node_trans(fni ni)
{
    ntrans nt;

    nt = pmalloco(ni->p,sizeof(_ntrans));
    nt->sessions = xhash_new(509);
    nt->waiting = xhash_new(64);
    nt->delay = j_atoi(xmlnode_get_data(node_config(ni,"delay")),60);

    node_register(ni,PACKET_DELIVER,node_trans_deliver,(void *) nt);
    node_register(ni,PACKET_FARM,node_trans_farm,(void *) nt);

    register_beat(30,node_clear,(void *) nt);
}
